/*-
 * Copyright 2022,2023 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Nils Goroll <nils.goroll@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * iterate over a body and call callback for anything which looks like a url
 */

typedef void zfr_iter_f(void *priv, const char *u, const char *h);

enum zfr_iter_form_e {
	FRMST_KEY = 0,	// discarding, looking for =
	FRMST_VAL,	// in value after =
	FRMST_PERC,	// % seen
	FRMST_PERC1	// %x seen, high in hex
};

struct zfr_iter_form_s {
	enum zfr_iter_form_e	state;
	unsigned char		hex;
};

struct zfr_iter_s {
	unsigned		magic;
#define ZFR_ITER_MAGIC		0x2d560bd8
	unsigned		end_seen:1;
	struct zipflow_top	*zft;
	char			*buf;
	size_t			len;
	zfr_iter_f		*func;
	void			*priv;
	struct zfr_iter_form_s	form;
};

void zfr_iter_fini(struct zfr_iter_s *zis);
int zfr_iter_ws(void *, unsigned flush,
    const void *ptr, ssize_t len);
int zfr_iter_form(void *, unsigned flush,
    const void *ptr, ssize_t len);
